
#include "weapon/hornetgun.h"

const int MaxPlayers = 32;

bool HasHornetGun		[ MaxPlayers + 1 ];
bool HornetGunActive	[ MaxPlayers + 1 ];
bool HornetGunShouldDrop[ MaxPlayers + 1 ];

int HornetPuff;
int HornetTrail;

enum hgun_e 
{
	HGUN_IDLE1 = 0,
	HGUN_FIDGETSWAY,
	HGUN_FIDGETSHAKE,
	HGUN_DOWN,
	HGUN_UP,
	HGUN_SHOOT
};

#ifdef __linux__
	//void Precache( void* pWeapon )
	RageHookReturn Precache( void*& pWeapon )
#else
	void _fastcall Precache( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "Precache - START" );

		PRECACHE_MODEL( "models/v_alt_hgun_gflip.mdl" );
		PRECACHE_MODEL( "models/w_hgun.mdl" );
		PRECACHE_MODEL( "models/p_hgun.mdl" );

		PRECACHE_MODEL( "models/hornet.mdl" );

		PRECACHE_SOUND( "agrunt/ag_fire1.wav" );
		PRECACHE_SOUND( "agrunt/ag_fire2.wav" );
		PRECACHE_SOUND( "agrunt/ag_fire3.wav" );

		PRECACHE_SOUND( "hornet/ag_buzz1.wav" );
		PRECACHE_SOUND( "hornet/ag_buzz2.wav" );
		PRECACHE_SOUND( "hornet/ag_buzz3.wav" );

		PRECACHE_SOUND( "hornet/ag_hornethit1.wav" );
		PRECACHE_SOUND( "hornet/ag_hornethit2.wav" );
		PRECACHE_SOUND( "hornet/ag_hornethit3.wav" );

		PRECACHE_GENERIC( "sprites/weapon_hivehand.txt" );
		PRECACHE_GENERIC( "sprites/320hud1_valve.spr" );
		PRECACHE_GENERIC( "sprites/320hud2_valve.spr" );
		PRECACHE_GENERIC( "sprites/640hud2_valve.spr" );
		PRECACHE_GENERIC( "sprites/640hud5_valve.spr" );
		PRECACHE_GENERIC( "sprites/640hud7_valve.spr" );
		PRECACHE_GENERIC( "sprites/crosshairs_valve.spr" );

		HornetPuff = PRECACHE_MODEL( "sprites/muz1.spr" );
		HornetTrail = PRECACHE_MODEL( "sprites/laserbeam.spr" );

		//Rage->write_to_file_ln( "debug.txt", "Precache - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//int GetItemInfo( void* pWeapon, void* pItem )
	RageHookReturn GetItemInfo( void*& pWeapon, void*& pItem, int& ret )
#else
	int _fastcall GetItemInfo( void* pWeapon, DUMMY, void* pItem )
#endif
	{
		entvars_t*	v = PrivateToEntvars( pWeapon );
		ItemInfo*	p = ( ItemInfo* )pItem;

		p->pszName		= STRING( v->classname );
		p->pszAmmo1		= HORNET_CLASSNAME;
		p->iMaxAmmo1	= HORNET_MAX_CARRY;
		p->pszAmmo2		= NULL;
		p->iMaxAmmo2	= HORNETGUN_SECONDARY_MAX_AMMO;
		p->iMaxClip		= WEAPON_NOCLIP;
		p->iSlot		= HORNETGUN_SLOT_ID;
		p->iPosition	= HORNETGUN_POSITION;
		p->iId			= WEAPON_HORNETGUN;
		p->iFlags		= HORNETGUN_FLAGS;
		p->iWeight		= HORNETGUN_WEIGHT;

		set_pdata_int( ENT( v ), m_iId_, WEAPON_HORNETGUN );

		#ifdef __linux__
			ret = 1;
			return RageHookSupercede;
		#else
			return 1;
		#endif
	}


#ifdef __linux__
	//void FirstSpawn( void* pWeapon )
	RageHookReturn FirstSpawn( void*& pWeapon )
#else
	void _fastcall FirstSpawn( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "FirstSpawn - START" );

		entvars_t*	pevWeapon	= PrivateToEntvars( pWeapon );
		edict_t*	pentWeapon	= ENT( pevWeapon );

		pevWeapon->classname = MAKE_STRING( new_weapon );

		SET_MODEL( pentWeapon, "models/w_hgun.mdl" );

		set_pdata_int( pentWeapon, m_iId_, WEAPON_HORNETGUN );
		set_pdata_int( pentWeapon, m_iDefaultAmmo_, HORNETGUN_DEFAULT_GIVE );

		#ifdef __linux__
			FallInit( pWeapon );
		#else 
			FallInit( pWeapon, DUMMY_VAL );
		#endif

		//Rage->write_to_file_ln( "debug.txt", "FirstSpawn - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//int OnDeploy( void* pWeapon )
	RageHookReturn OnDeploy( void*& pWeapon, int& ret )
#else
	int _fastcall OnDeploy( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "Deploy - START" );

		edict_t* pentWeapon = PrivateToEdict( pWeapon );
		void *pPlayer = get_pdata_cbase( pentWeapon, m_pPlayer_ );

		//Rage->write_to_file_ln( "debug.txt", "\tDeploy - pWeapon = %x, pentWeapon = %x, pPlayer = %x (offset : %d)", pWeapon, pentWeapon, pPlayer, m_pPlayer_ );

		if( pPlayer == NULL )
		{
			//Rage->write_to_file_ln( "debug.txt", "\tDeploy - pPlayer NULL" );

			#ifdef __linux__
				ret = FALSE;
				return RageHookSupercede;
			#else
				return FALSE;
			#endif
		}

		set_pdata_float( pentWeapon, m_flNextPrimaryAttack_, 9999.0 );
		set_pdata_float( pentWeapon, m_flNextPrimaryAttack_, 9999.0 );
		set_pdata_float( pentWeapon, m_flNextAttack_, 9999.0 );

		//Rage->write_to_file_ln( "debug.txt", "\tDeploy - Before DefaultDeploy" );

		#ifdef __linux__
			int retdd = DefaultDeploy( pWeapon, "models/v_alt_hgun_gflip.mdl", "models/p_hgun.mdl", HGUN_UP, "hive", FALSE );
		#else 
			int retdd = DefaultDeploy( pWeapon, DUMMY_VAL, "models/v_alt_hgun_gflip.mdl", "models/p_hgun.mdl", HGUN_UP, "hive", FALSE );
		#endif

		//Rage->write_to_file_ln( "debug.txt", "\tDeploy - After DefaultDeploy - retdd = %d", retdd );

		if( retdd )
		{
			int playerIndex = Rage->private_to_index( pPlayer );
			
			if( MF_IsPlayerValid( playerIndex ) )
			{
				HornetGunActive[ playerIndex ] = true;
				HornetGunShouldDrop[ playerIndex ] = false;
			}
		}

		//Rage->write_to_file_ln( "debug.txt", "Deploy - END" );

		#ifdef __linux__
			ret = retdd;
			return RageHookSupercede;
		#else
			return retdd;
		#endif
	}


#ifdef __linux__
	//void Holster( void* pWeapon, int skiplocal )
	RageHookReturn Holster( void*& pWeapon, int& skiplocal )
#else
	void _fastcall Holster( void* pWeapon, DUMMY, int skiplocal )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "Holster - START" );

		edict_t *pentWeapon = PrivateToEdict( pWeapon );
		void *pPlayer = get_pdata_cbase( pentWeapon, m_pPlayer_ );

		int ownerIndex = Rage->private_to_index( pPlayer );

		if( !MF_IsPlayerValid( ownerIndex ) )
		#ifdef __linux__
			return RageHookSupercede;
		#else
			return;
		#endif

		HornetGunActive[ ownerIndex ] = false;

		edict_t* pentPlayer = PrivateToEdict( pPlayer );

		set_pdata_float( pentPlayer, m_flNextAttack_, 0.5 );

		#ifdef __linux__
			// SendWeaponAnim( pWeapon, HGUN_DOWN, FALSE );
		#else 
			// SendWeaponAnim( pWeapon, DUMMY_VAL, HGUN_DOWN, FALSE );
		#endif

		int ammoType	= get_pdata_int( pentWeapon, m_iPrimaryAmmoType_ );
		int ammo		= get_pdata_int( pentPlayer, m_rgAmmo_ + ammoType );

		if( !ammo )
		{
			set_pdata_int( pentPlayer, m_rgAmmo_ + ammoType, 1 );
		}

		//Rage->write_to_file_ln( "debug.txt", "Holster - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//void Reload( void* pWeapon )
	RageHookReturn Reload( void*& pWeapon )
#else
	void _fastcall Reload( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "Reload - START" );

		edict_t*	pentWeapon	= PrivateToEdict( pWeapon );
		void*		pPlayer		= get_pdata_cbase( pentWeapon, m_pPlayer_ );
		int			ownerIndex	= Rage->private_to_index( pPlayer );

		if( !MF_IsPlayerAlive( ownerIndex ) || !HornetGunActive[ ownerIndex ] )
		#ifdef __linux__
			return RageHookSupercede;
		#else
			return;
		#endif;

		edict_t*	pentPlayer	= PrivateToEdict( pPlayer );
		entvars_t*	pevWeapon	= EdictToEntvar( pentWeapon );

		int ammoType	= get_pdata_int( pentWeapon, m_iPrimaryAmmoType_ );
		int ammo		= get_pdata_int( pentPlayer, m_rgAmmo_ + ammoType );

		if( ammo >= HORNET_MAX_CARRY )
		#ifdef __linux__
			return RageHookSupercede;
		#else
			return;
		#endif

		while( ammo < HORNET_MAX_CARRY && RechargeTime < gpGlobals->time )
		{
			ammo++;
			RechargeTime = RechargeTime + 0.5;
		}

		set_pdata_int( pentPlayer, m_rgAmmo_ + ammoType, ammo );

		//Rage->write_to_file_ln( "debug.txt", "Reload - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//void PrimaryAttack( void* pWeapon )
	RageHookReturn PrimaryAttack( void*& pWeapon )
#else
	void _fastcall PrimaryAttack( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "PrimaryAttack - START" );

		edict_t*	pentWeapon	= PrivateToEdict( pWeapon );
		void*		pPlayer		= get_pdata_cbase( pentWeapon, m_pPlayer_ );

		if( pPlayer != NULL )
		{
			int ownerIndex = Rage->private_to_index( pPlayer );

			if( !MF_IsPlayerAlive( ownerIndex ) || !HornetGunActive[ ownerIndex ] )
			#ifdef __linux__
				return RageHookSupercede;
			#else
				return;
			#endif
		}

		#ifdef __linux__
			OnReload( pWeapon );
		#else 
			Reload( pWeapon, DUMMY_VAL );
		#endif

		entvars_t*	pevWeapon	= EdictToEntvar( pentWeapon );
		entvars_t*	pevPlayer	= PrivateToEntvars( pPlayer );
		edict_t*	pentPlayer	= ENT( pevPlayer );

		int ammoType	= get_pdata_int( pentWeapon, m_iPrimaryAmmoType_ );
		int ammo		= get_pdata_int( pentPlayer, m_rgAmmo_ + ammoType );

		if( ammo <= 0 )
		{
			#ifdef __linux__
				return RageHookSupercede;
			#else
				return;
			#endif
		}

		MAKE_VECTORS( pevPlayer->v_angle );

		#ifdef __linux__
			Vector gunPosition = GetGunPosition( pPlayer );
			Vector vecSrc = gunPosition + gpGlobals->v_forward * 16 + gpGlobals->v_right * 8 + gpGlobals->v_up * -12;
			entvars_t* pevHornet = Create( "info_target", vecSrc, pevPlayer->v_angle, pentPlayer );
		#else
			Vector gunPosition;
			GetGunPosition( pPlayer, DUMMY_VAL, &gunPosition );

			entvars_t* pevHornet = Create( "info_target", gunPosition + gpGlobals->v_forward * 16 + gpGlobals->v_right * 8 + gpGlobals->v_up * -12, pevPlayer->v_angle, pentPlayer );
		#endif

		pevHornet->velocity = gpGlobals->v_forward * 300;

		RechargeTime = gpGlobals->time + 0.5;

		ammo--;
		set_pdata_int( pentPlayer, m_rgAmmo_ + ammoType, ammo );
		set_pdata_int( pentPlayer, m_iWeaponVolume_, QUIET_GUN_VOLUME );
		set_pdata_int( pentPlayer, m_iWeaponFlash_ , DIM_GUN_FLASH );

		pevPlayer->punchangle.x = 2;

		#ifdef __linux__
			SendWeaponAnim( pWeapon, HGUN_SHOOT, FALSE );
		#else 
			SendWeaponAnim( pWeapon, DUMMY_VAL, HGUN_SHOOT, FALSE );
		#endif

		switch( RANDOM_LONG( 0, 2 ) )
		{
			case 0:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire1.wav", 1, ATTN_NORM, 0, 100 ); break;
			case 1:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire2.wav", 1, ATTN_NORM, 0, 100 ); break;
			case 2:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire3.wav", 1, ATTN_NORM, 0, 100 ); break;
		}

		#ifdef __linux__
			SetAnimation( pWeapon, PLAYER_ATTACK1 );
		#else 
			SetAnimation( pWeapon, DUMMY_VAL, PLAYER_ATTACK1 );
		#endif

		NextPrimaryAttack = NextPrimaryAttack + 0.25;

		if( NextPrimaryAttack < gpGlobals->time )
		{
			NextPrimaryAttack = gpGlobals->time + 0.25;
		}

		TimeWeaponIdle = gpGlobals->time + RANDOM_FLOAT( 10, 15 );

		//Rage->write_to_file_ln( "debug.txt", "PrimaryAttack - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//void SecondaryAttack( void* pWeapon )
	RageHookReturn SecondaryAttack( void*& pWeapon )
#else
	void _fastcall SecondaryAttack( void* pWeapon, DUMMY )
#endif
	{
		//Rage->write_to_file_ln( "debug.txt", "SecondaryAttack - START" );

		edict_t*	pentWeapon	= PrivateToEdict( pWeapon );
		void*		pPlayer		= get_pdata_cbase( pentWeapon, m_pPlayer_ );

		if( pPlayer != NULL )
		{
			int ownerIndex = Rage->private_to_index( pPlayer );

			if( !MF_IsPlayerAlive( ownerIndex ) || !HornetGunActive[ ownerIndex ] )
			#ifdef __linux__
				return RageHookSupercede;
			#else
				return;
			#endif
		}

		#ifdef __linux__
			OnReload( pWeapon );
		#else 
			Reload( pWeapon, DUMMY_VAL );
		#endif

		entvars_t*	pevWeapon	= EdictToEntvar( pentWeapon );
		entvars_t*	pevPlayer	= PrivateToEntvars( pPlayer );
		edict_t*	pentPlayer	= ENT( pevPlayer );

		int ammoType = get_pdata_int( pentWeapon, m_iPrimaryAmmoType_ );
		int ammo	 = get_pdata_int( pentPlayer, m_rgAmmo_ + ammoType );

		if( ammo <= 0 )
		{
			#ifdef __linux__
				return RageHookSupercede;
			#else
				return;
			#endif
		}

		MAKE_VECTORS( pevPlayer->v_angle );

		#ifdef __linux__
			Vector vecSrc = GetGunPosition( pPlayer ) + gpGlobals->v_forward * 16 + gpGlobals->v_right * 8 + gpGlobals->v_up * -12;
		#else
			Vector gunPosition;
			GetGunPosition( pPlayer, DUMMY_VAL, &gunPosition );

			Vector vecSrc = gunPosition + gpGlobals->v_forward * 16 + gpGlobals->v_right * 8 + gpGlobals->v_up * -12;
		#endif

		pevWeapon->impulse++;

		switch( pevWeapon->impulse )
		{
			case 1:
				vecSrc = vecSrc + gpGlobals->v_up * 8;
				break;
			case 2:
				vecSrc = vecSrc + gpGlobals->v_up * 8;
				vecSrc = vecSrc + gpGlobals->v_right * 8;
				break;
			case 3:
				vecSrc = vecSrc + gpGlobals->v_right * 8;
				break;
			case 4:
				vecSrc = vecSrc + gpGlobals->v_up * -8;
				vecSrc = vecSrc + gpGlobals->v_right * 8;
				break;
			case 5:
				vecSrc = vecSrc + gpGlobals->v_up * -8;
				break;
			case 6:
				vecSrc = vecSrc + gpGlobals->v_up * -8;
				vecSrc = vecSrc + gpGlobals->v_right * -8;
				break;
			case 7:
				vecSrc = vecSrc + gpGlobals->v_right * -8;
				break;
			case 8:
				vecSrc = vecSrc + gpGlobals->v_up * 8;
				vecSrc = vecSrc + gpGlobals->v_right * -8;
				pevWeapon->impulse = 0;
				break;
		}

		entvars_t* pevHornet = Create( "info_target", vecSrc, pevPlayer->v_angle, pentPlayer );

		pevHornet->velocity = gpGlobals->v_forward * 1200;
		pevHornet->angles = UTIL_VecToAngles( pevHornet->velocity );

		SetThink_( ENT( pevHornet ), ( void* )&StartDart );

		RechargeTime = gpGlobals->time + 0.5;

		ammo--;
		set_pdata_int( pentPlayer, m_rgAmmo_ + ammoType, ammo );
		set_pdata_int( pentPlayer, m_iWeaponVolume_, NORMAL_GUN_VOLUME );
		set_pdata_int( pentPlayer, m_iWeaponFlash_ , DIM_GUN_FLASH );

		pevPlayer->punchangle.x = 2;

		#ifdef __linux__
			SendWeaponAnim( pWeapon, HGUN_SHOOT, FALSE );
		#else 
			SendWeaponAnim( pWeapon, DUMMY_VAL, HGUN_SHOOT, FALSE );
		#endif

		switch( RANDOM_LONG( 0, 2 ) )
		{
			case 0:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire1.wav", 1, ATTN_NORM, 0, 100 ); break;
			case 1:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire2.wav", 1, ATTN_NORM, 0, 100 ); break;
			case 2:	EMIT_SOUND_DYN2( pentPlayer, CHAN_WEAPON, "agrunt/ag_fire3.wav", 1, ATTN_NORM, 0, 100 ); break;
		}

		#ifdef __linux__
			SetAnimation( pWeapon, PLAYER_ATTACK1 );
		#else 
			SetAnimation( pWeapon, DUMMY_VAL, PLAYER_ATTACK1 );
		#endif

		NextPrimaryAttack	= gpGlobals->time + 0.1;
		NextSecondaryAttack	= gpGlobals->time + 0.1;
		TimeWeaponIdle		= gpGlobals->time + RANDOM_FLOAT( 10, 15 );

		//Rage->write_to_file_ln( "debug.txt", "SecondaryAttack - END" );

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}


#ifdef __linux__
	//void WeaponIdle( void* pWeapon )
	RageHookReturn WeaponIdle( void*& pWeapon )
#else
	void _fastcall WeaponIdle( void* pWeapon, DUMMY )
#endif
	{
		entvars_t*	pevWeapon	= PrivateToEntvars( pWeapon );
		edict_t*	pentWeapon	= ENT( pevWeapon );

		void *pPlayer =  get_pdata_cbase( pentWeapon, m_pPlayer_ );

		if( pPlayer != NULL )
		{
			int ownerIndex = Rage->private_to_index( pPlayer );

			if( !MF_IsPlayerAlive( ownerIndex ) || !HornetGunActive[ ownerIndex ] )
			#ifdef __linux__
				return RageHookSupercede;
			#else
				return;
			#endif
		}

		#ifdef __linux__
			OnReload( pWeapon );
		#else 
			Reload( pWeapon, DUMMY_VAL );
		#endif

		if( TimeWeaponIdle > gpGlobals->time )
		#ifdef __linux__
			return RageHookSupercede;
		#else
			return;
		#endif

		int animation;
		float rand = RANDOM_FLOAT( 0, 1 );

		if( rand <= 0.75 )
		{
			animation = HGUN_IDLE1;
			TimeWeaponIdle = gpGlobals->time + 30.0 / 16 * (2);
		}
		else if( rand <= 0.875 )
		{
			animation = HGUN_FIDGETSWAY;
			TimeWeaponIdle = gpGlobals->time + 40.0 / 16.0;
		}
		else
		{
			animation = HGUN_FIDGETSHAKE;
			TimeWeaponIdle = gpGlobals->time + 35.0 / 16.0;
		}

		#ifdef __linux__
			SendWeaponAnim( pWeapon, animation, FALSE );
		#else 
			SendWeaponAnim( pWeapon, DUMMY_VAL, animation, FALSE );
		#endif

		#ifdef __linux__
			return RageHookSupercede;
		#endif
	}



RageHookReturn New_AddToPlayer_Post( void*& pItem, void*& pPlayer, int& ret )
{
	//Rage->write_to_file_ln( "debug.txt", "New_AddToPlayer_Post - item : %s, ret = %d - START", STRING( PrivateToEntvars( pItem )->classname ), ret );

	if( !ret )
		return RageHookIgnored;

	entvars_t*	pevItem		= PrivateToEntvars( pItem );
	edict_t*	pentItem	= ENT( pevItem );
	int			playerIndex = Rage->private_to_index( pPlayer );

	if( MF_IsPlayerAlive( playerIndex ) )
	{
		HasHornetGun[ playerIndex ] = true;
		HornetGunShouldDrop[ playerIndex ] = false;

		set_pdata_int( pentItem, m_iPrimaryAmmoType_ , 15 );

		entvars_t* pevPlayer = PrivateToEntvars( pPlayer );

		int ammoType = get_pdata_int( pentItem, m_iPrimaryAmmoType_ );

		set_pdata_int( ENT( pevPlayer ), m_rgAmmo_ + ammoType, HORNET_MAX_CARRY );
		
		set_pdata_int( pentItem, m_iClip_, -1 );
		//set_pdata_int( pentItem, m_iClientClip_, -1 );

		int ammoIndex = GetAmmoIndex( HORNET_CLASSNAME );	

		/*MESSAGE_BEGIN( MSG_ONE, MsgIdCurWeapon, NULL, pevPlayer );
			WRITE_BYTE( 1 );
			WRITE_BYTE( WEAPON_HORNETGUN );
			WRITE_BYTE( -1 );
		MESSAGE_END();*/

		MESSAGE_BEGIN( MSG_ONE, MsgIdWeaponList, NULL, pevPlayer );  
			WRITE_STRING( new_weapon );					// WeaponName  
			WRITE_BYTE( ammoIndex );					// PrimaryAmmoID  
			WRITE_BYTE( HORNET_MAX_CARRY );				// PrimaryAmmoMaxAmount  
			WRITE_BYTE( HORNETGUN_SECONDARY_AMMO_ID );	// SecondaryAmmoID  
			WRITE_BYTE( HORNETGUN_SECONDARY_MAX_AMMO );	// SecondaryAmmoMaxAmount  
			WRITE_BYTE( HORNETGUN_SLOT_ID );			// SlotID (0...N)  
			WRITE_BYTE( HORNETGUN_POSITION );			// NumberInSlot (1...N)  
			WRITE_BYTE( WEAPON_HORNETGUN );				// WeaponID  
			WRITE_BYTE( HORNETGUN_FLAGS );				// Flags  
		MESSAGE_END();

		//Rage->write_to_file_ln( "debug.txt", "\tNew_AddToPlayer_Post - Message sent" );
	}

	//Rage->write_to_file_ln( "debug.txt", "New_AddToPlayer_Post - END" );

	return RageHookIgnored;
}

RageHookReturn Base_AddToPlayer_Post( void*& pItem, void*& pPlayer, int& ret )
{
	//Rage->write_to_file_ln( "debug.txt", "Base_AddToPlayer_Post - START - ret = %d", ret );

	if( !ret )
		return RageHookIgnored;

	int	playerIndex = Rage->private_to_index( pPlayer );

	if( MF_IsPlayerAlive( playerIndex ) ) // filter me ?
	{
		entvars_t* pevPlayer = PrivateToEntvars( pPlayer );

		MESSAGE_BEGIN( MSG_ONE, MsgIdWeaponList, NULL, pevPlayer );  
			WRITE_STRING( base_weapon );			// WeaponName  
			WRITE_BYTE( AK47_PRIMARY_AMMO_ID );		// PrimaryAmmoID  
			WRITE_BYTE( AK47_PRIMARY_MAX_AMMO );	// PrimaryAmmoMaxAmount  
			WRITE_BYTE( AK47_SECONDARY_AMMO_ID );	// SecondaryAmmoID  
			WRITE_BYTE( AK47_SECONDARY_MAX_AMMO );	// SecondaryAmmoMaxAmount  
			WRITE_BYTE( AK47_SLOT_ID );				// SlotID (0...N)  
			WRITE_BYTE( AK47_POSITION );			// NumberInSlot (1...N)  
			WRITE_BYTE( WEAPON_AK47 );				// WeaponID  
			WRITE_BYTE( 0 );						// Flags  
		MESSAGE_END(); 
	}

	//Rage->write_to_file_ln( "debug.txt", "Base_AddToPlayer_Post - END" );

	return RageHookIgnored;
}

RageHookReturn PreThink( void*& pPlayer )
{
	int playerIndex = Rage->private_to_index( pPlayer );

	if( !MF_IsPlayerAlive( playerIndex ) || !HornetGunActive[ playerIndex ] )
		return RageHookIgnored;

	entvars_t*	pevPlayer	= PrivateToEntvars( pPlayer );
	edict_t*	pentPlayer	= ENT( pevPlayer );

	if( pevPlayer->button & ( IN_ATTACK | IN_ATTACK2 ) )
	{
		void *pItem = get_pdata_cbase( pentPlayer, m_pActiveItem_ );

		if( pItem )
		{
			entvars_t *pevWeapon = PrivateToEntvars( pItem );

			if( pevPlayer->button & IN_ATTACK && NextPrimaryAttack < gpGlobals->time )
			{
				#ifdef __linux__
					OnPrimaryAttack( pItem );
				#else 
					PrimaryAttack( pItem, DUMMY_VAL );
				#endif		
			}
			else if( pevPlayer->button & IN_ATTACK2 && NextSecondaryAttack < gpGlobals->time )
			{
				#ifdef __linux__
					OnSecondaryAttack( pItem );
				#else 
					SecondaryAttack( pItem, DUMMY_VAL );
				#endif		
			}
		}
	}

	return RageHookIgnored;
}


RageHookReturn AddPlayerItem_Pre( void*& pPlayer, void*& pItem, int& ret )
{
	entvars_t*	pevItem		= PrivateToEntvars( pItem );
	int			playerIndex	= Rage->private_to_index( pPlayer );
	entvars_t*  pevPlayer	= Rage->index_to_entvar( playerIndex );

	//Rage->write_to_file_ln( "debug.txt", "AddPlayerItem_Pre : item = %s, alive = %d - START", STRING( pevItem->classname ), MF_IsPlayerAlive( playerIndex ) );

	if( !MF_IsPlayerAlive( playerIndex ) )
		return RageHookIgnored;

	if( pevPlayer->weapons & ( 1 << WEAPON_AK47 ) && !strcmp( STRING( pevItem->classname ), new_weapon ) )
	{
		//Rage->write_to_file_ln( "debug.txt", "\tAddPlayerItem_Pre - Has already an ak47" );

		#ifdef __linux__
			DropPlayerItem( pPlayer, base_weapon );
		#else 
			DropPlayerItem( pPlayer, DUMMY_VAL, base_weapon );
		#endif
	}
	else if( HasHornetGun[ playerIndex ] && !strcmp( STRING( pevItem->classname ), base_weapon ) )
	{
		//Rage->write_to_file_ln( "debug.txt", "\tAddPlayerItem_Pre - Has already an hornet gun" );
		ret = FALSE;
		return RageHookSupercede;
	}

	//Rage->write_to_file_ln( "debug.txt", "AddPlayerItem_Pre - END" );

	return RageHookIgnored;
}

void SetModel( edict_t *e, const char *m )
{
	entvars_t* pevWeapon = EdictToEntvar( e );
	edict_t* pOwner = pevWeapon->owner;

	//Rage->write_to_file_ln( "debug.txt", "SetModel : model = %s, owner = %d - START", m, pOwner ? Rage->entvar_to_index( EdictToEntvar( pOwner ) ) : 0 );

	if( !pOwner  )
		RETURN_META( MRES_IGNORED );

	int ownerIndex = Rage->entvar_to_index( EdictToEntvar( pOwner ) );

	if( MF_IsPlayerValid( ownerIndex ) && HornetGunShouldDrop[ ownerIndex ] && !strcmp( m, AK47_WORLD_MODEL ) )
	{
		HornetGunShouldDrop[ ownerIndex ] = false;

		SET_MODEL( e, HORNETGUN_WORLD_MODEL );

		RETURN_META( MRES_SUPERCEDE );
	}

	//Rage->write_to_file_ln( "debug.txt", "SetModel - END" );

	RETURN_META( MRES_IGNORED; );
}

RageHookReturn OnDropPlayerItem( void*& pPlayer, char*& itemName )
{
	//Rage->write_to_file_ln( "debug.txt", "OnDropPlayerItem - START" );

	int playerIndex = Rage->private_to_index( pPlayer );

	void* pActiveItem;
	bool shouldDrop;

	if( !strlen( itemName ) )
	{
		pActiveItem = get_pdata_cbase( PrivateToEdict( pPlayer ), m_pActiveItem_ );

		if( pActiveItem )
			shouldDrop = !strcmp( STRING( PrivateToEntvars( pActiveItem )->classname ), new_weapon );
	}
	else
	{
		shouldDrop = !strcmp( itemName, new_weapon );
	}

	HornetGunShouldDrop[ playerIndex ] = false;

	if( shouldDrop && MF_IsPlayerValid( playerIndex ) && HornetGunActive[ playerIndex ] )
	{
		HornetGunShouldDrop[ playerIndex ] = true;
		HasHornetGun	[ playerIndex ] = false;
		HornetGunActive	[ playerIndex ] = false;
	}

	//Rage->write_to_file_ln( "debug.txt", "OnDropPlayerItem - END" );

	return RageHookIgnored;
}

RageHookReturn Killed( void *& pVictim, entvars_t*& pevAttacker, int& iGib )
{
	//Rage->write_to_file_ln( "debug.txt", "Killed - START" );

	int playerIndex = Rage->private_to_index( pVictim );

	if( MF_IsPlayerValid( playerIndex ) )
	{
		HornetGunShouldDrop[ playerIndex ] = true;
		HornetGunActive	[ playerIndex ] = false;
		HasHornetGun	[ playerIndex ] = false;
	}

	//Rage->write_to_file_ln( "debug.txt", "Killed - END" );

	return RageHookIgnored;
}

qboolean ClientConnect( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] )
{
	int playerIndex = ENTINDEX( pEntity );

	if( MF_IsPlayerValid( playerIndex ) )
	{
		HornetGunShouldDrop[ playerIndex ] = false;
		HasHornetGun	[ playerIndex ] = false;
		HornetGunActive	[ playerIndex ] = false;
	}

	RETURN_META_VALUE( MRES_IGNORED, FALSE );
}

void ClientDisconnect( edict_t *pEntity )
{
	int playerIndex = ENTINDEX( pEntity );

	if( MF_IsPlayerValid( playerIndex ) )
	{
		HornetGunShouldDrop[ playerIndex ] = true;
		HasHornetGun	[ playerIndex ] = false;
		HornetGunActive	[ playerIndex ] = false;
	}

	RETURN_META( MRES_IGNORED );
}

RageHookReturn OnGetAmmoIndex( char const*& ammoName, int& ret )
{
	if( ammoName && !strcmp( ammoName, HORNET_CLASSNAME ) )
	{
		ret = HORNETGUN_PRIMARY_AMMO_ID;
		return RageHookOverride;
	}

	return RageHookIgnored;
}