
#include "weapon_util.h"
#include "weapon_config.h"
#include <weapon\projectile\hornet.h>

entvars_t* Create( char* classname, Vector& vecOrigin, Vector& vecAngles, edict_t* pentOwner )
{
	//Rage->write_to_file_ln( "debug.txt", "Create - class = %s - START", classname );

	edict_t	*pent = CREATE_NAMED_ENTITY( ALLOC_STRING( classname ) );

	if( FNullEnt( pent ) )
	{
		ALERT( at_console, "NULL Ent in Create!\n" );
		return NULL;
	}

	entvars_t* pev = Rage->index_to_entvar( Rage->private_to_index( GET_PRIVATE( pent ) ) );

	pev->owner	= pentOwner;
	pev->origin	= vecOrigin;
	pev->angles	= vecAngles;
	pev->iuser1	= 1337;

	MDLL_Spawn( ENT( pev ) );

	//Rage->write_to_file_ln( "debug.txt", "Create - END" );

	return pev;
}

int UTIL_EntitiesInBox( void **pList, int listMax, const Vector &mins, const Vector &maxs, int flagMask )
{
	//Rage->write_to_file_ln( "debug.txt", "UTIL_EntitiesInBox - START" );

	edict_t*	pEdict = g_engfuncs.pfnPEntityOfEntIndex( 1 );
	void*		pEntity;
	int			count;

	count = 0;

	if( !pEdict )
		return count;

	for( int i = 1; i < gpGlobals->maxEntities; i++, pEdict++ )
	{
		if( pEdict->free )	// Not in use
			continue;

		if( flagMask && !( pEdict->v.flags & flagMask ) )	// Does it meet the criteria?
			continue;

		if( mins.x > pEdict->v.absmax.x ||
			mins.y > pEdict->v.absmax.y ||
			mins.z > pEdict->v.absmax.z ||
			maxs.x < pEdict->v.absmin.x ||
			maxs.y < pEdict->v.absmin.y ||
			maxs.z < pEdict->v.absmin.z )
			continue;

		pEntity = Rage->index_to_private( Rage->entvar_to_index( EdictToEntvar( pEdict ) ) );

		if( !pEntity )
			continue;

		pList[ count ] = pEntity;
		count++;

		if( count >= listMax )
			return count;
	}

	//Rage->write_to_file_ln( "debug.txt", "UTIL_EntitiesInBox - END" );

	return count;
}

BOOL IsInViewCone( entvars_t *pevHornet, entvars_t *pevEntity )
{
	Vector2D	vec2LOS;
	float		flDot;

	MAKE_VECTORS( pevHornet->angles );

	vec2LOS = ( pevEntity->origin - pevHornet->origin ).Make2D();
	vec2LOS = vec2LOS.Normalize();

	flDot = DotProduct( vec2LOS, gpGlobals->v_forward.Make2D() );

	if( flDot >= HORNET_FOV )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}

};

BOOL FVisible( entvars_t *pevHornet, entvars_t *pevEntity )
{ 
	TraceResult tr;
	Vector		vecLookerOrigin;
	Vector		vecTargetOrigin;

	if( FBitSet( pevEntity->flags, FL_NOTARGET ) )
		return FALSE;

	if( ( pevHornet->waterlevel != 3 && pevEntity->waterlevel == 3 ) 
		|| ( pevHornet->waterlevel == 3 && pevEntity->waterlevel == 0 ) )
		return FALSE;

	vecLookerOrigin = pevHornet->origin + pevHornet->view_ofs;
	vecTargetOrigin = pevEntity->origin + pevEntity->view_ofs;

	TRACE_LINE( vecLookerOrigin, vecTargetOrigin, ignore_monsters | 0x100, ENT( pevHornet ), &tr);

	if( tr.flFraction != 1.0 )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

void* BestVisibleEnemy( void* pHornet, int distance )
{
	//Rage->write_to_file_ln( "debug.txt", "BestVisibleEnemy - START" );

	entvars_t*	pevHornet	= PrivateToEntvars( pHornet );
	edict_t*	pentHornet	= ENT( pevHornet );
	int			OwnerIndex	= Rage->entvar_to_index( EdictToEntvar( pevHornet->owner ) );

	void *pList[ 100 ];
	void *pSightEnt = NULL;
	entvars_t *pevSightEnt = NULL;

	Vector delta = Vector( distance, distance, distance );

	int count = UTIL_EntitiesInBox( pList, 100, pevHornet->origin - delta, pevHornet->origin + delta, FL_CLIENT | FL_MONSTER );

	void *pReturn = NULL;
	int	iNearest = 8192;
	int	iDist;

	for( int i = 0; i < count; i++ )
	{
		pSightEnt = pList[i];
		pevSightEnt = PrivateToEntvars( pSightEnt );

		if( !pevSightEnt->modelindex )
			continue;

		if( pSightEnt == pHornet || pevSightEnt->modelindex == pevHornet->modelindex  )
			continue;

		if( pevSightEnt->pContainingEntity == pevHornet->owner || pevSightEnt->health <= 0)
			continue;

		if( !IsInViewCone( pevHornet, pevSightEnt ) && !FBitSet( pevSightEnt->flags, FL_NOTARGET ) && FVisible( pevHornet, pevSightEnt ) )
			continue;

		if( !( pevSightEnt->deadflag == DEAD_NO && pevSightEnt->health > 0 ) )
			continue;

		if( MF_ExecuteForward( OnHornetTracking, Rage->private_to_index( pSightEnt ), OwnerIndex, Rage->entvar_to_index( pevSightEnt ) ) )
			continue;

		iDist = ( pevSightEnt->origin - pevHornet->origin ).Length();

		if( iDist <= iNearest )
		{
			iNearest = iDist;
			pReturn = pSightEnt;
		}
	}

	//Rage->write_to_file_ln( "debug.txt", "BestVisibleEnemy - END" );

	return pReturn;
}

Vector UTIL_VecToAngles( const Vector &vec )
{
	float rgflVecOut[3];
	VEC_TO_ANGLES( vec, rgflVecOut );
	return Vector( rgflVecOut );
}